@@ -12,5 +12,5 @@ class RedshiftDatabase(PostgreSQLDatabase):
     query_cls = RedshiftQuery
 
     def __init__(self, host='localhost', port=5439, database=None,
-                 user=None, password=None, **kwags):
-        super(RedshiftDatabase, self).__init__(host, port, database, user, password, **kwags)
+                 user=None, password=None, **kwargs):
+        super(RedshiftDatabase, self).__init__(host, port, database, user, password, **kwargs)
