@@ -4,7 +4,6 @@ from fireant.queries.builder import (
     DataSetBlenderQueryBuilder,
     DimensionChoicesQueryBuilder,
 )
-from fireant.queries.builder.query_builder import validate_fields
 from fireant.utils import (
     deepcopy,
     immutable,
@@ -71,7 +70,8 @@ class DataSetBlender:
         self.dimension_map = dimension_map
 
         # Wrap all dataset fields with another field on top so that:
-        #   1. DataSetBlender doesn't share a reference to a field with a DataSet
+        #   1. DataSetBlender doesn't share a reference to a field with a DataSet (__hash__ is used to find out which
+        #      dataset the field is in - see the Field class' __hash__ method for more details)
         #   2. When complex fields are added, the `definition` attribute will always have at least one field within
         #      its object graph
         self.fields = DataSet.Fields(
@@ -88,11 +88,23 @@ class DataSetBlender:
         self.latest = self.primary_dataset.latest
         self.annotation = None
 
+    @property
+    def return_additional_metadata(self) -> bool:
+        # When using data blending, datasets are nested inside DataSetBlender objects. Additionally,
+        # the primary_dataset can be a combination of datasets depending on how many datasets are being blended.
+        # This helper property walks the tree to return the return_additional_metadata value from the original
+        # primary dataset.
+        dataset = self.primary_dataset
+        while not isinstance(dataset, DataSet):
+            dataset = dataset.primary_dataset
+
+        return dataset.return_additional_metadata
+
     def __eq__(self, other):
         return isinstance(other, DataSetBlender) and self.fields == other.fields
 
     def __repr__(self):
-        return "BlendedDatSet(fields=[{}])".format(
+        return "BlendedDataSet(fields=[{}])".format(
             ",".join([repr(f) for f in self.fields])
         )
 
@@ -135,6 +147,11 @@ class DataSetBlenderBuilder:
         )
 
     def on_dimensions(self):
+        """
+        This function doesn't work when blending more than 2 datasets. It won't select dimensions in the 3rd dataset
+        and further. self.primary_dataset might be a DataSetBlender object itself. We would want to dig deeper until
+        we find the actual primary dataset.
+        """
         dimension_map = {}
 
         for secondary_ds_field in self.secondary_dataset.fields:
