@@ -1,14 +1,19 @@
 from collections import Iterable
 from enum import Enum
 from functools import wraps
+from numbers import Number
+from typing import Type, Union
 
+from pypika import Field as PyPikaField
 from pypika.enums import Arithmetic
 from pypika.terms import (
     ArithmeticExpression,
+    Function,
     Mod,
     Node,
     Pow,
 )
+
 from .filters import (
     AntiPatternFilter,
     BooleanFilter,
@@ -86,16 +91,28 @@ class Field(Node):
         A precision value for rounding decimals. By default, no rounding will be applied.
 
     :param prefix: (optional)
-        A prefix for rendering labels in visualizations such as '$'
+        A prefix for rendering labels in visualizations such as '$'.
 
-    :param suffix:
-        A suffix for rendering labels in visualizations such as '€'
+    :param suffix: (optional)
+        A suffix for rendering labels in visualizations such as '€'.
+
+    :param thousands: (optional)
+        A character to be used as a thousand separator when rendering values.
+
+    :param hyperlink_template: (optional)
+        A url string that can also reference other fields in a dataset. To reference a field put its alias
+        between {}. This is only used in certain widgets (i.e. react table).
+
+    :param fetch_only: (optional)
+        Whether the field data should be ignored in widgets. This is useful for not displaying hyperlink
+        dependencies, which might be necessary only for the purpose of generating a hyperlink and have no
+        effect on the dimension grouping.
     """
 
     def __init__(
         self,
-        alias,
-        definition,
+        alias: str,
+        definition: Union[PyPikaField, Type[Function]],
         data_type: DataType = DataType.number,
         label=None,
         hint_table=None,
@@ -104,6 +121,7 @@ class Field(Node):
         thousands: str = None,
         precision: int = None,
         hyperlink_template: str = None,
+        fetch_only: bool = False,
     ):
         self.alias = alias
         self.data_type = data_type
@@ -115,35 +133,58 @@ class Field(Node):
         self.thousands = thousands
         self.precision = precision
         self.hyperlink_template = hyperlink_template
+        self.fetch_only = fetch_only
+
+        # An artificial field is created dynamically as the query is mounted, instead of being defined during
+        # instantiation. That's the case for set dimensions, for instance. The only practical aspect of this
+        # is that artificial dimensions are hashed differently (see dunder hash method). Instead of using the
+        # memory reference, artificial dimensions use a set of its fields.
+        self.is_artificial = False
 
     @property
     def is_aggregate(self):
         return self.definition.is_aggregate
 
-    def eq(self, other):
+    @property
+    def groupable(self):
+        """
+        Whether this field should be allowed to be specified
+        in a query group by statement
+        """
+        return not self.is_aggregate
+
+    @property
+    def is_wrapped(self) -> bool:
+        """
+        This allows calling code can easily tell whether the field has been wrapped.
+        This occurs if data blending is being used.
+        """
+        return bool(getattr(self.definition, 'definition', False))
+
+    def eq(self, other: Number) -> ComparatorFilter:
         return ComparatorFilter(self, ComparisonOperator.eq, other)
 
-    def ne(self, other):
+    def ne(self, other: Number) -> ComparatorFilter:
         return ComparatorFilter(self, ComparisonOperator.ne, other)
 
     @restrict_types(CONTINUOUS_TYPES)
-    def gt(self, other):
+    def gt(self, other: Number) -> ComparatorFilter:
         return ComparatorFilter(self, ComparisonOperator.gt, other)
 
     @restrict_types(CONTINUOUS_TYPES)
-    def ge(self, other):
+    def ge(self, other: Number) -> ComparatorFilter:
         return ComparatorFilter(self, ComparisonOperator.gte, other)
 
     @restrict_types(CONTINUOUS_TYPES)
-    def lt(self, other):
+    def lt(self, other: Number):
         return ComparatorFilter(self, ComparisonOperator.lt, other)
 
     @restrict_types(CONTINUOUS_TYPES)
-    def le(self, other):
+    def le(self, other: Number) -> ComparatorFilter:
         return ComparatorFilter(self, ComparisonOperator.lte, other)
 
     @restrict_types(CONTINUOUS_TYPES)
-    def between(self, lower, upper):
+    def between(self, lower, upper) -> RangeFilter:
         """
         Creates a filter to filter a dataset query.
 
@@ -157,7 +198,7 @@ class Field(Node):
         """
         return RangeFilter(self, lower, upper)
 
-    def isin(self, values: Iterable):
+    def isin(self, values: Iterable) -> ContainsFilter:
         """
         Creates a filter that filters to rows where
 
@@ -170,7 +211,7 @@ class Field(Node):
         """
         return ContainsFilter(self, values)
 
-    def notin(self, values):
+    def notin(self, values: Iterable) -> ExcludesFilter:
         """
         Creates a filter to filter a dataset query.
 
@@ -185,15 +226,15 @@ class Field(Node):
         return ExcludesFilter(self, values)
 
     @restrict_types([DataType.text])
-    def like(self, pattern, *patterns):
+    def like(self, pattern: str, *patterns: Iterable) -> PatternFilter:
         return PatternFilter(self, pattern, *patterns)
 
     @restrict_types([DataType.text])
-    def not_like(self, pattern, *patterns):
+    def not_like(self, pattern: str, *patterns: Iterable) -> AntiPatternFilter:
         return AntiPatternFilter(self, pattern, *patterns)
 
     @restrict_types([DataType.boolean])
-    def is_(self, value: bool):
+    def is_(self, value: bool) -> BooleanFilter:
         """
         Creates a filter to filter a dataset query.
 
@@ -204,7 +245,7 @@ class Field(Node):
         """
         return BooleanFilter(self, value)
 
-    def void(self):
+    def void(self) -> VoidFilter:
         return VoidFilter(self)
 
     def __eq__(self, other):
@@ -271,16 +312,37 @@ class Field(Node):
             """
             return hash(None)
 
+        if self.is_artificial:
+            """
+            Python's deep copy built-in are a problem when:
+                Handling artificial fields (e.g. set field) in data blending.
+            Given artificial fields are dynamically generated, they can't be properly compared when hashed
+            by memory id. Therefore we hash them differently.
+            """
+            return hash((self.alias, self.label, self.data_type, self.definition))
+
         return id(self)
 
     def for_(self, replacement):
         return replacement
 
-    def get_sql(self, **kwargs):
-        raise NotImplementedError
+    def get_sql(self, *args, **kwargs) -> str:
+        return self.definition.get_sql(*args, **kwargs)
 
     @property
     @immutable
     def share(self):
         self._share = True
         return self
+
+
+def is_metric_field(field: Field) -> bool:
+    """
+    Returns whether a field is a metric.
+
+    :param field: A Field instance.
+    :return: A boolean.
+    """
+    # Only terms wrapped with aggregate functions, such as SUM, will evaluate is_aggregate to True in Pypika.
+    # This is a good way to validate that the set dimension, in question, is actually encompassing a metric.
+    return field.is_aggregate
