@@ -1,5 +1,4 @@
 import math
-import re
 from datetime import (
     date,
     datetime,
@@ -60,20 +59,31 @@ def return_none(value):
 
 
 @filter_kwargs
-def _format_decimal(value, thousands="", precision=None):
+def _format_decimal(value, thousands="", precision=None, suffix=None, use_raw_value=False):
     if not isinstance(value, (int, float)):
         return value
 
-    precision_pattern = (
-        "{{:{}.{}f}}".format(thousands, precision)
-        if precision is not None
-        else "{{:{}f}}".format(thousands)
-    )
+    if use_raw_value and suffix == '%':
+        # When raw values are required, we divide percentage values by 100 to ensure they
+        # work well with Spreadsheet applications like Excel.
+        value /= 100
 
-    f_value = precision_pattern.format(value)
+        # Add extra precision to offset the division
+        if precision is not None:
+            precision += 2
+
+    if use_raw_value:
+        precision_pattern = f'{{:.{precision if precision is not None else 16}f}}'
+    elif precision is not None:
+        precision_pattern = f'{{:{thousands}.{precision}f}}'
+    else:
+        precision_pattern = f'{{:{thousands}f}}'
+
+    value = precision_pattern.format(value)
     if precision is None:
-        f_value = f_value.rstrip("0").rstrip(".")
-    return f_value
+        value = value.rstrip('0').rstrip('.')
+
+    return value
 
 
 @filter_kwargs
@@ -91,9 +101,6 @@ def _identity(value):
     return value
 
 
-UNSAFE_CHARS = re.compile(r"[^\w\d\s\-:()]")
-
-
 def json_value(value):
     """
     This function will return only values safe for JSON
@@ -115,12 +122,11 @@ def safe_value(value):
     if value in TOTALS_MARKERS:
         return TOTALS_VALUE
 
-    str_value = (
+    return (
         date_as_string(value, interval_key="iso")
         if isinstance(value, date)
         else str(value)
     )
-    return UNSAFE_CHARS.sub("$", str_value)
 
 
 @filter_kwargs
@@ -185,6 +191,14 @@ def display_value(
         The dataset field that the value represents.
     :param date_as:
         A format function for datetimes.
+    :param nan_value:
+        The value to return if the value is a Pandas null (np.nan) value.
+    :param null_value:
+        The value to return if the value is None
+    :param use_raw_value:
+        Do not output a value with prefix/suffixes. If a suffix is a percentage sign and this value is true, the raw
+        value will be value/100 to make it easier for users when exporting to tools like Excel.
+
     :return:
         A formatted string containing the display value for the metric.
     """
