@@ -1,3 +1,5 @@
+from typing import TYPE_CHECKING, Union
+
 from pypika import Order
 
 from fireant.dataset.fields import Field
@@ -12,7 +14,9 @@ from ..sets import (
     apply_set_dimensions,
     omit_set_filters,
 )
-from ...dataset.modifiers import DimensionModifier
+
+if TYPE_CHECKING:
+    from fireant.dataset import DataSet
 
 
 class QueryException(DataSetException):
@@ -64,15 +68,21 @@ class QueryBuilder(object):
     via a set of functions which can be chained together.
     """
 
-    def __init__(self, dataset, table):
+    def __init__(self, dataset: 'DataSet'):
+        """
+        :param dataset: DataSet to build the query for
+        """
         self.dataset = dataset
-        self.table = table
+        self.table = dataset.table
         self._dimensions = []
         self._filters = []
         self._orders = None
-        self._limit = None
-        self._offset = None
+        self._client_limit = None
+        self._client_offset = None
+        self._query_limit = None
+        self._query_offset = None
 
+    # noinspection PyDefaultArgument
     def __deepcopy__(self, memodict={}):
         fields = [d for d in self._dimensions]
         if self._orders is not None:
@@ -132,28 +142,54 @@ class QueryBuilder(object):
             self._orders += [(field, orientation)]
 
     @immutable
-    def limit(self, limit):
+    def limit_query(self, limit):
         """
         Sets the limit of the query.
 
         :param limit:
             A limit on the number of database rows returned.
         :return:
-            A copy of the query with the limit set.
+            A copy of the query with the query limit set.
         """
-        self._limit = limit
+        self._query_limit = limit
 
     @immutable
-    def offset(self, offset):
+    def offset_query(self, offset):
         """
         Sets the offset of the query.
 
         :param offset:
             A offset on the number of database rows returned.
         :return:
-            A copy of the query with the ofset set.
+            A copy of the query with the query offset set.
         """
-        self._offset = offset
+        self._query_offset = offset
+
+    @immutable
+    def limit_client(self, limit):
+        """
+        Sets the limit to be used when paginating the Pandas Dataframe after the results
+        have been returned
+
+        :param limit:
+            A limit on the number of dataframe rows returned.
+        :return:
+            A copy of the query with the dataframe limit set.
+        """
+        self._client_limit = limit
+
+    @immutable
+    def offset_client(self, offset):
+        """
+        Sets the offset to be used when paginating the Pandas Dataframe after the results
+        have been returned
+
+        :param offset:
+            A offset on the number of dataframe rows returned
+        :return:
+            A copy of the query with the dataframe offset set.
+        """
+        self._client_offset = offset
 
     @property
     def dimensions(self):
@@ -168,7 +204,7 @@ class QueryBuilder(object):
 
         :return: A list of Field instances.
         """
-        return apply_set_dimensions(self._dimensions, self._filters)
+        return apply_set_dimensions(self._dimensions, self._filters, self.dataset)
 
     @property
     def filters(self):
@@ -186,7 +222,7 @@ class QueryBuilder(object):
 
         :return: None or a list of tuples shaped as Field instance and ordering.
         """
-        return self._orders
+        return self._orders or self.default_orders
 
     @property
     def default_orders(self):
@@ -228,7 +264,22 @@ class QueryBuilder(object):
         """
         queries = add_hints(self.sql, hint)
 
-        return fetch_data(self.dataset.database, queries, self.dimensions)
+        max_rows_returned, data = fetch_data(self.dataset.database, queries, self.dimensions)
+        return self._transform_for_return(data, max_rows_returned=max_rows_returned)
+
+    def _apply_pagination(self, query):
+        # Some platforms require an order by when pagination is used. Therefore, if there is no ordering set,
+        # we just default to the first column.
+        if not self.orders:
+            query = query.orderby(1)
+
+        query = query.limit(min(self._query_limit or float('inf'), self.dataset.database.max_result_set_size))
+        return query.offset(self._query_offset)
+
+    def _transform_for_return(self, widget_data, **metadata) -> Union[dict, list]:
+        return dict(data=widget_data, metadata=dict(**metadata)) \
+            if self.dataset.return_additional_metadata \
+            else widget_data
 
 
 class ReferenceQueryBuilderMixin:
