@@ -1,25 +1,20 @@
-from functools import reduce
-from typing import (
-    Iterable,
-    Sized,
-    Union,
-)
+import logging
+from typing import Iterable, List, Tuple, Type
 
 import pandas as pd
+from pypika.queries import QueryBuilder
 
 from fireant.database import Database
-from fireant.dataset.fields import Field, DataType
+from fireant.dataset.fields import DataType, Field
 from fireant.dataset.references import calculate_delta_percent
 from fireant.dataset.totals import get_totals_marker_for_dtype
-
-from fireant.utils import (
-    alias_selector,
-    chunks,
-)
-from .finders import find_totals_dimensions, find_field_in_modified_field
+from fireant.utils import alias_selector, chunks
+from .finders import find_field_in_modified_field, find_totals_dimensions
 from .pandas_workaround import df_subtract
 from ..dataset.modifiers import RollupValue
 
+logger = logging.getLogger(__name__)
+
 # Passing in an empty dictionary as format option to pandas' parse_dates causes errors to be ignored instead of
 # being coerced into NaT.
 PANDAS_TO_DATETIME_FORMAT = {}
@@ -27,17 +22,12 @@ PANDAS_TO_DATETIME_FORMAT = {}
 
 def fetch_data(
     database: Database,
-    queries: Union[Sized, Iterable],
+    queries: List[Type[QueryBuilder]],
     dimensions: Iterable[Field],
     share_dimensions: Iterable[Field] = (),
     reference_groups=(),
-):
-    queries = [
-        str(
-            query.limit(min(query._limit or float("inf"), database.max_result_set_size))
-        )
-        for query in queries
-    ]
+) -> Tuple[int, pd.DataFrame]:
+    queries = [str(query) for query in queries]
 
     # Indicate which dimensions need to be parsed as date types
     # For this we create a dictionary with the dimension alias as key and PANDAS_TO_DATETIME_FORMAT as value
@@ -48,8 +38,10 @@ def fetch_data(
             pandas_parse_dates[alias_selector(unmodified_dimension.alias)] = PANDAS_TO_DATETIME_FORMAT
 
     results = database.fetch_dataframes(*queries, parse_dates=pandas_parse_dates)
+    max_rows_returned = max([len(x) for x in results], default=0)
+    logger.info('max_rows_returned', extra={'row_count': max_rows_returned, 'database': str(database)})
 
-    return reduce_result_set(results, reference_groups, dimensions, share_dimensions)
+    return max_rows_returned, reduce_result_set(results, reference_groups, dimensions, share_dimensions)
 
 
 def reduce_result_set(
@@ -105,16 +97,16 @@ def reduce_result_set(
         # The data frames will be ordered so that the first group will contain the data without any rolled up
         # dimensions, then followed by the groups with them, ordered by the last rollup dimension first.
         if totals_dimension_keys[:i]:
-            merged_df = _replace_nans_for_totals_values(merged_df, dimension_dtypes)
+            merged_df = _replace_rollup_constants_for_totals_markers(merged_df, dimension_dtypes)
 
         group_data_frames.append(merged_df)
 
     return pd.concat(group_data_frames, sort=False).sort_index(na_position="first")
 
 
-def _replace_nans_for_totals_values(data_frame, dtypes):
-    # some things are just easier to do without an index. Reset it temporarily to replace NaN values with the rollup
-    # marker values
+def _replace_rollup_constants_for_totals_markers(data_frame, dtypes):
+    # some things are just easier to do without an index. Reset it temporarily to replace Rollup constants with the
+    # rollup marker values
     index_names = data_frame.index.names
     data_frame.reset_index(inplace=True)
 
