@@ -1,10 +1,11 @@
+from typing import Tuple
+
 import pandas as pd
 from pandas.core.dtypes.common import is_datetime64_ns_dtype
+from pypika import Order
 
 from fireant.utils import alias_selector
 
-from pypika import Order
-
 
 def _get_window(limit, offset):
     start = offset
@@ -12,7 +13,7 @@ def _get_window(limit, offset):
     return start, end
 
 
-def _apply_sorting(orders):
+def _get_sorting_schema(orders) -> Tuple[list, bool]:
     sort_values, ascending = zip(
         *[
             (alias_selector(field.alias), orientation is Order.asc)
@@ -25,35 +26,38 @@ def _apply_sorting(orders):
 def paginate(data_frame, widgets, orders=(), limit=None, offset=None):
     """
     :param data_frame:
-        The result set to paginate.
+        The result set to sort.
     :param widgets:
-        An iterable of widgets that the pagination is being applied for.
+        An iterable of widgets that the sort is being applied for.
     :param orders:
         An iterable of (<Dimension/Metric>, pypika.Order)
-    :param limit:
+     :param limit:
         A limit of the number of data points/series
     :param offset:
         A offset of the number of data points/series
     :return:
         A paginated data frame. If the widget required grouped pagination, then there should be an upperbound
         `limit*(n_index_level_0)`. Otherwise the data frame should have the same length as the limit.
+    :return:
+        A sorted data frame.
     """
     if len(data_frame) == 0:
         return data_frame
 
     start, end = _get_window(limit, offset)
-    group_pagination = isinstance(data_frame.index, pd.MultiIndex) and any(
-        [getattr(widget, "group_pagination", False) for widget in widgets]
+
+    needs_group_pagination = isinstance(data_frame.index, pd.MultiIndex) and any(
+        [getattr(widget, 'group_pagination', False) for widget in widgets]
     )
 
-    if group_pagination:
+    if needs_group_pagination:
         return _group_paginate(data_frame, start, end, orders)
     return _simple_paginate(data_frame, start, end, orders)
 
 
 def _simple_paginate(data_frame, start=None, end=None, orders=()):
     """
-    Applies pagination which limits the number of rows in the data frame.
+    Applies pagination which limits the number of rows in the dataframe.
 
     :param data_frame:
         A data frame to paginate
@@ -66,7 +70,7 @@ def _simple_paginate(data_frame, start=None, end=None, orders=()):
         and a pypika.Order.
     """
     if orders:
-        sort, ascending = _apply_sorting(orders)
+        sort, ascending = _get_sorting_schema(orders)
         data_frame = data_frame.sort_values(by=sort, ascending=ascending)
 
     return data_frame[start:end]
@@ -94,7 +98,6 @@ def _group_paginate(data_frame, start=None, end=None, orders=()):
     """
     Applies pagination which limits the number of rows in the data frame grouped by the zeroth index level. This will
     in turn paginate the number of series in the data frame.
-
     :param data_frame:
         A data frame to paginate
     :param start:
@@ -118,14 +121,12 @@ def _group_paginate(data_frame, start=None, end=None, orders=()):
 
     if orders:
         aggregated_df = dimension_groups.aggregate(_aggregate_dimension_groups)
-        sort, ascending = _apply_sorting(orders)
+        sort, ascending = _get_sorting_schema(orders)
         sorted_df = aggregated_df.sort_values(by=sort, ascending=ascending)
         sorted_dimension_values = tuple(sorted_df.index)[start:end]
 
     else:
-        sorted_dimension_values = tuple(dimension_groups.apply(lambda g: g.name))[
-            start:end
-        ]
+        sorted_dimension_values = tuple(dimension_groups.apply(lambda g: g.name))[start:end]
 
     sorted_dimension_values = (
         pd.Index(sorted_dimension_values, name=dimension_levels[0])
@@ -134,19 +135,25 @@ def _group_paginate(data_frame, start=None, end=None, orders=()):
     )
 
     def _apply_pagination(df):
-        # This function applies sorting by using the sorted dimension values as an index to select values in the right
-        # order out of the data frame. The index must be filtered to only values that are in this data frame, since
-        # there might be missing combinations of index values.
+        """
+        This function applies sorting by using the sorted dimension values as an index to select values in the right
+        order out of the data frame. The index must be filtered to only values that are in this data frame, since
+        there might be missing combinations of index values.
+        """
         dfx = df.reset_index(level=0, drop=True)
         value_in_index = sorted_dimension_values.isin(dfx.index)
         index_slice = sorted_dimension_values[value_in_index].values
 
         """
-        In the case of bool dimensions, convert index_slice to an array of literal `True`, because pandas `.loc` handles
-        lists of bool as a mask.
+        In the case of bool dimensions, convert index_slice to an array of literal `True` and/or `False`,
+        because pandas `.loc` handles lists of bool as a mask. In which `True` returns the row and `False` won't.
         """
         if bool in {type(x) for x in sorted_dimension_values}:
-            index_slice |= True
+            num_missing_entries_in_boolean_mask = len(index_slice)
+            index_slice = [False]*len(dfx.index)
+
+            for i in range(num_missing_entries_in_boolean_mask):
+                index_slice[i] = True
 
         # Need to include nulls so append them to the end of the sorted data frame
         isnull = _index_isnull(dfx)
