@@ -143,11 +143,7 @@ def getdeepattr(d, keys, default_value=None):
     d_level = d
 
     for key in keys:
-        if (
-            isinstance(d_level, dict)
-            and key not in d_level
-            or not hasattr(d_level, key)
-        ):
+        if key not in d_level:
             return default_value
 
         d_level = d_level[key]
@@ -289,3 +285,7 @@ def listify(func):
         return retval
 
     return new_func
+
+
+def filter_nones(iterable):
+    return filter(None.__ne__, iterable)
