@@ -1,9 +1,12 @@
+import itertools
 from datetime import timedelta
 
-import itertools
 import pandas as pd
 
-from fireant import formats, utils
+from fireant import (
+    formats,
+    utils,
+)
 from fireant.dataset.fields import DataType
 from fireant.dataset.totals import TOTALS_MARKERS
 from fireant.reference_helpers import (
@@ -12,8 +15,12 @@ from fireant.reference_helpers import (
     reference_prefix,
     reference_suffix,
 )
+from fireant.utils import alias_selector
 from .base import TransformableWidget
-from .chart_base import ChartWidget, ContinuousAxisSeries
+from .chart_base import (
+    ChartWidget,
+    ContinuousAxisSeries,
+)
 
 DEFAULT_COLORS = (
     "#DDDF0D",
@@ -46,6 +53,7 @@ MARKER_SYMBOLS = ("circle", "square", "diamond", "triangle", "triangle-down")
 
 SERIES_NEEDING_MARKER = (ChartWidget.LineSeries, ChartWidget.AreaSeries)
 TS_UPPER_BOUND = pd.Timestamp.max - timedelta(seconds=1)
+ALWAYS_SHARED_TOOLTIP_CHART_TYPES = {ChartWidget.PieSeries.type}
 
 
 def has_only_line_series(axis):
@@ -57,20 +65,24 @@ class HighCharts(ChartWidget, TransformableWidget):
     group_pagination = True
 
     def __init__(
-        self, title=None, colors=None, x_axis_visible=True, tooltip_visible=True
+        self,
+        title=None,
+        colors=None,
+        x_axis_visible=True,
+        tooltip_visible=True,
+        split_dimension=None,
     ):
         super(HighCharts, self).__init__()
         self.title = title
         self.colors = colors or DEFAULT_COLORS
         self.x_axis_visible = x_axis_visible
         self.tooltip_visible = tooltip_visible
+        self.split_dimension = split_dimension or None
 
     def __repr__(self):
         return ".".join(["HighCharts()"] + [repr(axis) for axis in self.items])
 
-    def transform(
-        self, data_frame, dimensions, references, annotation_frame=None
-    ):
+    def transform(self, data_frame, dimensions, references, annotation_frame=None):
         """
         - Main entry point -
 
@@ -87,31 +99,86 @@ class HighCharts(ChartWidget, TransformableWidget):
         :param annotation_frame:
             A data frame containing annotation data.
         :return:
-            A dict meant to be dumped as JSON.
+            A dict or a list of dicts meant to be dumped as JSON.
         """
+        result_df = data_frame.copy()
+
+        hide_dimensions = {
+            dimension for dimension in dimensions if dimension.fetch_only
+        }
+        self.hide_data_frame_indexes(result_df, hide_dimensions)
+
+        dimension_map = {
+            alias_selector(dimension.alias): dimension for dimension in dimensions
+        }
+
+        render_group = []
+        split_dimension = self.split_dimension
+
+        if split_dimension:
+            split_dimension_alias = alias_selector(split_dimension.alias)
+
+            categories = self._categories(
+                result_df, dimension_map, split_dimension_alias,
+            )
+
+            for category in categories:
+                render_group.append(
+                    [
+                        result_df.xs(
+                            category, level=split_dimension_alias, drop_level=False
+                        ),
+                        formats.display_value(category, split_dimension) or category,
+                    ]
+                )
+
+        if not render_group:
+            render_group = [(result_df, None)]
+
+        num_charts = len(render_group)
+
+        charts = [
+            self._render_individual_chart(
+                chart_df,
+                dimensions,
+                references,
+                annotation_frame=annotation_frame,
+                titleSuffix=titleSuffix,
+                num_charts=num_charts,
+            )
+            for chart_df, titleSuffix in render_group
+        ]
+
+        return charts[0] if num_charts == 1 else charts
+
+    def _render_individual_chart(
+        self,
+        data_frame,
+        dimensions,
+        references,
+        annotation_frame=None,
+        titleSuffix="",
+        num_charts=1,
+    ):
+        result_df = data_frame
+
+        dimension_map = {
+            alias_selector(dimension.alias): dimension for dimension in dimensions
+        }
+
         colors = itertools.cycle(self.colors)
 
         is_timeseries = dimensions and dimensions[0].data_type == DataType.date
 
-        dimension_map = {
-            dimension_alias: dimension
-            for dimension_alias, dimension in zip(data_frame.index.names, dimensions)
-        }
-        num_dimensions = len(dimension_map)
-        dimension_fields = [
-            dimension_map[field_alias]
-            for field_alias in data_frame.index.names[:num_dimensions]
-        ]
-
         # Timestamp.max is used as a marker for rolled up dimensions (totals). Filter out the totals value for the
         # dimension used for the x-axis
         if is_timeseries and len(data_frame) > 0:
-            data_frame = self._remove_date_totals(data_frame)
+            result_df = self._remove_date_totals(result_df)
 
         # Group the results by index levels after the 0th, one for each series
         # This will result in a series for every combination of dimension values and each series will contain a data set
         # across the 0th dimension (used for the x-axis)
-        series_data_frames = self._group_by_series(data_frame)
+        series_data_frames = self._group_by_series(result_df)
 
         total_num_series = sum([len(axis) for axis in self.items])
 
@@ -134,19 +201,31 @@ class HighCharts(ChartWidget, TransformableWidget):
                 axis_idx,
                 axis_color,
                 colors,
-                data_frame,
+                result_df,
                 series_data_frames,
-                dimension_fields,
+                dimensions,
                 references,
                 is_timeseries,
             )
 
-        x_axis = self._render_x_axis(data_frame, dimensions, dimension_map)
+        categories = self._categories(result_df, dimension_map)
+        x_axis = self._render_x_axis(dimensions, categories)
 
         annotations = []
         if has_only_line_series(axis) and annotation_frame is not None:
             annotations = self._render_annotation(annotation_frame, x_axis)
 
+        extra = {}
+
+        if num_charts > 1:
+            extra["title"] = {
+                "text": f"{self.title} ({titleSuffix})" if self.title else titleSuffix
+            }
+            extra["chart"] = {
+                # Height of 240px is the smallest we can have, while still fully displaying the chart menu.
+                "height": 240
+            }
+
         return {
             "title": {"text": self.title},
             "xAxis": x_axis,
@@ -154,44 +233,68 @@ class HighCharts(ChartWidget, TransformableWidget):
             "colors": self.colors,
             "series": series,
             "tooltip": {
-                "shared": True,
                 "useHTML": True,
                 "enabled": self.tooltip_visible,
+                # When only a single datapoint per series is available, shared tooltips should be avoided.
+                # Since it looks clunky and often supersedes most of the chart.
+                "shared": all([
+                    len(item['data']) > 1 or item['type'] in ALWAYS_SHARED_TOOLTIP_CHART_TYPES
+                    for item in series
+                ]),
             },
             "legend": {"useHTML": True},
             "annotations": annotations,
+            **extra,
         }
 
-    def _render_x_axis(self, data_frame, dimensions, dimension_map):
+    def _categories(self, data_frame, dimension_map, dimension_alias=None):
+        is_mi = isinstance(data_frame.index, pd.MultiIndex)
+        levels = data_frame.index.levels if is_mi else [data_frame.index]
+
+        first_level = None
+
+        if dimension_alias:
+            level_index = 0
+
+            for i, level in enumerate(levels):
+                if level.name == dimension_alias:
+                    level_index = i
+                    break
+
+            first_level = levels[level_index]
+        else:
+            first_level = levels[0]
+
+        if first_level is not None and first_level.name is not None:
+            dimension_alias = first_level.name
+            dimension = dimension_map[dimension_alias]
+            return [
+                formats.display_value(category, dimension) or category
+                if not pd.isnull(category)
+                else None
+                for category in first_level
+            ]
+
+        return []
+
+    def _render_x_axis(self, dimensions, categories):
         """
         Renders the xAxis configuration.
 
         https://api.highcharts.com/highcharts/xAxis
 
-        :param data_frame:
         :param dimensions:
-        :param dimension_map:
+        :param categories:
         :return:
         """
-        is_mi = isinstance(data_frame.index, pd.MultiIndex)
-        first_level = data_frame.index.levels[0] if is_mi else data_frame.index
-
         is_timeseries = dimensions and dimensions[0].data_type == DataType.date
+
         if is_timeseries:
             return {"type": "datetime", "visible": self.x_axis_visible}
 
-        categories = ["All"]
-        if first_level.name is not None:
-            dimension_alias = first_level.name
-            dimension = dimension_map[dimension_alias]
-            categories = [
-                formats.display_value(category, dimension) or category
-                for category in first_level
-            ]
-
         return {
             "type": "category",
-            "categories": categories,
+            "categories": categories if categories else ["All"],
             "visible": self.x_axis_visible,
         }
 
@@ -239,7 +342,7 @@ class HighCharts(ChartWidget, TransformableWidget):
         colors,
         data_frame,
         series_data_frames,
-        dimension_fields,
+        dimensions,
         references,
         is_timeseries=False,
     ):
@@ -253,7 +356,7 @@ class HighCharts(ChartWidget, TransformableWidget):
         :param axis_color:
         :param colors:
         :param series_data_frames:
-        :param dimension_fields:
+        :param dimensions:
         :param references:
         :param is_timeseries:
         :return:
@@ -267,7 +370,7 @@ class HighCharts(ChartWidget, TransformableWidget):
                 for reference in [None] + references:
                     hc_series.append(
                         self._render_pie_series(
-                            series.metric, reference, data_frame, dimension_fields
+                            series.metric, reference, data_frame, dimensions
                         )
                     )
                 continue
@@ -280,7 +383,7 @@ class HighCharts(ChartWidget, TransformableWidget):
             ):
                 dimension_values = utils.wrap_list(dimension_values)
                 dimension_label = self._format_dimension_values(
-                    dimension_fields[1:], dimension_values
+                    dimensions[1:], dimension_values
                 )
 
                 hc_series += self._render_highcharts_series(
@@ -426,6 +529,12 @@ class HighCharts(ChartWidget, TransformableWidget):
     def _render_pie_series(self, metric, reference, data_frame, dimension_fields):
         metric_alias = utils.alias_selector(metric.alias)
 
+        if self.split_dimension:
+            dimension_fields = [
+                dimension for dimension in dimension_fields if dimension != self.split_dimension
+            ]
+            data_frame = data_frame.reset_index(alias_selector(self.split_dimension.alias), drop=True)
+
         data = []
         for dimension_values, y in data_frame[metric_alias].iteritems():
             dimension_values = utils.wrap_list(dimension_values)
@@ -564,8 +673,8 @@ class HighCharts(ChartWidget, TransformableWidget):
         return data_frame.groupby(level=levels, sort=False)
 
     @staticmethod
-    def _format_dimension_values(dimension_fields, dimension_values):
+    def _format_dimension_values(dimensions, dimension_values):
         return ", ".join(
-            str.strip(formats.display_value(value, dimension_field) or str(value))
-            for value, dimension_field in zip(dimension_values, dimension_fields)
+            str.strip(formats.display_value(value, dimension) or str(value))
+            for value, dimension in zip(dimension_values, dimensions)
         )
