@@ -1,7 +1,8 @@
-import pandas as pd
 from functools import partial
 from typing import Iterable
 
+import pandas as pd
+
 from fireant import formats
 from fireant.dataset.fields import Field
 from fireant.utils import alias_selector, wrap_list
@@ -16,6 +17,7 @@ class Pandas(TransformableWidget):
         metric: Field,
         *metrics: Iterable[Field],
         pivot=(),
+        hide=(),
         transpose=False,
         sort=None,
         ascending=None,
@@ -23,6 +25,7 @@ class Pandas(TransformableWidget):
     ):
         super(Pandas, self).__init__(metric, *metrics)
         self.pivot = pivot
+        self.hide = hide
         self.transpose = transpose
         self.sort = sort
         self.ascending = ascending
@@ -51,7 +54,11 @@ class Pandas(TransformableWidget):
         :param use_raw_values:
             Don't add prefix or postfix to values.
         """
-        result = data_frame.copy()
+        result_df = data_frame.copy()
+
+        dimension_aliases = [
+            alias_selector(dimension.alias) for dimension in dimensions
+        ]
 
         items = [
             item if reference is None else ReferenceItem(item, reference)
@@ -59,29 +66,51 @@ class Pandas(TransformableWidget):
             for reference in [None] + references
         ]
 
-        if isinstance(data_frame.index, pd.MultiIndex):
-            index_levels = [alias_selector(dimension.alias) for dimension in dimensions]
-            result = result.reorder_levels(index_levels)
+        if isinstance(result_df.index, pd.MultiIndex):
+            result_df = result_df.reorder_levels(dimension_aliases)
 
-        result = result[[alias_selector(item.alias) for item in items]]
+        result_df = result_df[[alias_selector(item.alias) for item in items]]
+
+        hide_dimensions = set(self.hide) | {
+            dimension for dimension in dimensions if dimension.fetch_only
+        }
+        self.hide_data_frame_indexes(result_df, hide_dimensions)
+
+        hide_aliases = {
+            dimension.alias for dimension in hide_dimensions
+        }
 
         if dimensions:
-            result.index.names = [
-                dimension.label or dimension.alias for dimension in dimensions
+            result_df.index.names = [
+                dimension.label or dimension.alias
+                for dimension in dimensions
+                if dimension.alias not in hide_aliases
             ]
 
-        result.columns = pd.Index([item.label for item in items], name="Metrics")
+        result_df.columns = pd.Index([item.label for item in items], name="Metrics")
 
         pivot_dimensions = [
-            dimension.label or dimension.alias for dimension in self.pivot
+            dimension.label or dimension.alias
+            for dimension in self.pivot
+            if dimension.alias not in hide_aliases
         ]
-        pivot_df = self.pivot_data_frame(result, pivot_dimensions, self.transpose)
-
-        return self.add_formatting(dimensions, items, pivot_df, use_raw_values).fillna(
+        result_df, _, _ = self.pivot_data_frame(result_df, pivot_dimensions, self.transpose)
+        return self.add_formatting(dimensions, items, result_df, use_raw_values).fillna(
             value=formats.BLANK_VALUE
         )
 
-    def pivot_data_frame(self, data_frame, pivot=(), transpose=False):
+    @staticmethod
+    def _should_data_frame_be_transformed(data_frame, pivot_dimensions, transpose):
+        if not pivot_dimensions and not transpose:
+            return False
+
+        if transpose and len(pivot_dimensions) == len(data_frame.index.names):
+            # Pivot and transpose cancel each other out
+            return False
+
+        return True
+
+    def pivot_data_frame(self, data_frame, pivot_dimensions, transpose):
         """
         Pivot and transpose the data frame. Dimensions including in the `pivot` arg will be unshifted to columns. If
         `transpose` is True the data frame will be transposed. If there is only index level in the data frame (ie. one
@@ -91,29 +120,30 @@ class Pandas(TransformableWidget):
 
         :param data_frame:
             The result set data frame
-        :param pivot:
+        :param pivot_dimensions:
             A list of index aliases for `data_frame` of levels to shift
         :param transpose:
             A boolean true or false whether to transpose the data frame.
         :return:
-            The shifted/transposed data frame
+            Tuple(The shifted/transposed data frame, is_pivoted, is_transposed)
         """
-        not_transforming_df = not (pivot or transpose)
-        pivot_and_transpose_cancel_out = transpose and len(pivot) == len(
-            data_frame.index.names
-        )
-        if not_transforming_df or pivot_and_transpose_cancel_out:
-            return self.sort_data_frame(data_frame)
+        is_pivoted = False
+        is_transposed = False
+
+        if not self._should_data_frame_be_transformed(data_frame, pivot_dimensions, transpose):
+            return self.sort_data_frame(data_frame), is_pivoted, is_transposed
 
         # NOTE: Don't pivot a single dimension data frame. This turns the data frame into a series and pivots the
         # metrics anyway. Instead, transpose the data frame.
-        should_transpose_instead_of_pivot = len(pivot) == len(data_frame.index.names)
+        should_transpose_instead_of_pivot = len(pivot_dimensions) == len(data_frame.index.names)
 
-        if pivot and not should_transpose_instead_of_pivot:
-            data_frame = data_frame.unstack(level=pivot)
+        if pivot_dimensions and not should_transpose_instead_of_pivot:
+            data_frame = data_frame.unstack(level=pivot_dimensions)
+            is_pivoted = True
 
         if transpose or should_transpose_instead_of_pivot:
             data_frame = data_frame.transpose()
+            is_transposed = True
 
         # If there are more than one column levels and the last level is a single metric, drop the level
         if isinstance(data_frame.columns, pd.MultiIndex) and 1 == len(
@@ -126,7 +156,7 @@ class Pandas(TransformableWidget):
                 0
             )  # drop the metrics level
 
-        return self.sort_data_frame(data_frame)
+        return self.sort_data_frame(data_frame), is_pivoted, is_transposed
 
     def sort_data_frame(self, data_frame):
         if not self.sort or len(data_frame) == 1:
@@ -166,7 +196,7 @@ class Pandas(TransformableWidget):
     def add_formatting(self, dimensions, items, pivot_df, use_raw_values):
         format_df = pivot_df.copy()
 
-        def _get_f_display(item):
+        def _get_field_display(item):
             return partial(
                 formats.display_value,
                 field=item,
@@ -181,25 +211,25 @@ class Pandas(TransformableWidget):
             and len(dimensions) == len(self.pivot) > 0
         ):
             for item in items:
-                f_display = _get_f_display(item)
+                field_display = _get_field_display(item)
                 format_df.loc[items[0].label] = format_df.loc[items[0].label].apply(
-                    f_display
+                    field_display
                 )
 
             return format_df
 
         if self.pivot and len(items) == 1:
-            f_display = _get_f_display(items[0])
-            format_df = format_df.applymap(f_display)
+            field_display = _get_field_display(items[0])
+            format_df = format_df.applymap(field_display)
             return format_df
 
         for item in items:
             key = item.label
-            f_display = _get_f_display(item)
+            field_display = _get_field_display(item)
             format_df[key] = (
-                format_df[key].apply(f_display)
+                format_df[key].apply(field_display)
                 if isinstance(format_df[key], pd.Series)
-                else format_df[key].applymap(f_display)
+                else format_df[key].applymap(field_display)
             )
 
         return format_df
