@@ -1,10 +1,16 @@
+import colorsys
 import re
 from collections import OrderedDict, defaultdict
 from functools import partial
 
 import pandas as pd
+import numpy as np
 
-from fireant.dataset.fields import DataType, Field
+from fireant.dataset.fields import (
+    DataType,
+    Field,
+)
+from fireant.dataset.filters import ComparisonOperator
 from fireant.dataset.totals import (
     DATE_TOTALS,
     NUMBER_TOTALS,
@@ -24,12 +30,12 @@ from fireant.reference_helpers import reference_alias
 from fireant.utils import (
     alias_for_alias_selector,
     alias_selector,
+    getdeepattr,
     setdeepattr,
     wrap_list,
 )
 from .base import ReferenceItem
 from .pandas import Pandas
-from fireant.dataset.filters import ComparisonOperator
 
 TOTALS_LABEL = "Totals"
 METRICS_DIMENSION_ALIAS = "metrics"
@@ -37,6 +43,25 @@ F_METRICS_DIMENSION_ALIAS = alias_selector(METRICS_DIMENSION_ALIAS)
 _display_value = partial(display_value, nan_value="", null_value="")
 
 
+def hex_to_rgb(hex_val):
+    """
+    https://stackoverflow.com/questions/29643352/converting-hex-to-rgb-value-in-python
+    """
+    return tuple(int(hex_val[i:i + 2], 16) for i in (0, 2, 4))
+
+
+def rgb_to_hex(rgb_val):
+    """
+    https://stackoverflow.com/questions/3380726/converting-a-rgb-color-tuple-to-a-six-digit-code-in-python
+    """
+    return "{0:02x}{1:02x}{2:02x}".format(*rgb_val)
+
+
+def hex_to_hsv(hex_color):
+    rgb_color = hex_to_rgb(hex_color)
+    return colorsys.rgb_to_hsv(*(val / 255 for val in rgb_color))
+
+
 class FormattingField:
     def __init__(self, metric=None, reference=None, operation=None):
         self.metric = metric
@@ -56,24 +81,115 @@ class FormattingField:
         return self.metric.alias
 
 
-class FormattingConditionRule:
-    def __init__(self, field, operator, value, color):
+class FormattingRule:
+    def __init__(self, field, color, covers_row):
         self.field = field
+        self.color = color
+        self.covers_row = covers_row
+
+    def applies(self, value):
+        return True
+
+    def determine_colors(self, value):
+        background_color = self._determine_background_color(value)
+        return background_color, self._determine_text_color(background_color)
+
+    def _determine_background_color(self, value):
+        return self.color
+
+    def _determine_text_color(self, background_color):
+        """
+        Determines, using the luminance of the background color, whether the text color should be light or dark.
+        """
+        rgb_bg_color = hex_to_rgb(background_color)
+        luminance = (0.299 * rgb_bg_color[0] + 0.587 * rgb_bg_color[1] + 0.114 * rgb_bg_color[2]) / 255;
+        return 'FDFDFD' if luminance < 0.5 else '212121';
+
+    def get_field_selector(self):
+        return alias_selector(self.field.get_alias())
+
+
+class FormattingConditionRule(FormattingRule):
+    def __init__(self, field, operator, value, color, covers_row=False):
+        super().__init__(field, color, covers_row)
         self.operator = operator
         self.value = value
-        self.color = color
 
-    def apply(self, value):
-        return self.color if ComparisonOperator.eval(value, self.operator, self.value) else None
+    def applies(self, value):
+        return ComparisonOperator.eval(value, self.operator, self.value)
 
 
-def apply_formatting_rules(rules, value):
-    color = None
+class FormattingHeatMapRule(FormattingRule):
+    """
+    The colors of the heatmap are calculated using the HSV color format.
+    The Hue (position in the color spectrum) and Value (brightness) of the final color are the same as the rule color.
+    The Saturation (color purity) of the final color depends on both the submitted value and the saturation of the rule
+    color. We calculate a saturation_spread which is either the saturation of the rule minus 0.05 or 0.50, whatever is
+    highest. This we will multiply with the ratio of the value compared to the min and max values which should be set
+    with set_min_max. The final value we increase with 0.05 to not have any white values.
+    This will result in for instance a final saturation range of [0.05; 1.00] if the rule color is very saturated
+    and in [0.05; 0.55] for a low saturated rule color.
+
+    If a start_color is supplied the same calculations are done for both the color and the start_color, but the
+    start_color will cover the first 50% of values and it will result in a color transition over white.
+    """
+
+    WHITE = 'FFFFFF'
+
+    def __init__(self, field, color, start_color=None, covers_row=False):
+        super().__init__(field, color, covers_row)
+        self.hsv_color = hex_to_hsv(self.color)
+        if start_color:
+            self.hsv_start_color = hex_to_hsv(start_color)
+            self.start_saturation_spread = max(0.50, self.hsv_start_color[1])
+        else:
+            self.hsv_start_color = None
+        self.saturation_spread = max(0.50, self.hsv_color[1] - 0.05)
+        self.min_val, self.value_range = None, None
+
+    def set_min_max(self, min_val, max_val):
+        if not self._is_invalid(min_val) and not self._is_invalid(max_val):
+            self.min_val = min_val
+            self.value_range = max_val - min_val
+
+
+    @staticmethod
+    def _is_invalid(val):
+        return pd.isna(val) or np.isinf(val)
+
+    @staticmethod
+    def get_hex_color_with_new_saturation(base_hsv_color, saturation):
+        calculated_hsv_color = colorsys.hsv_to_rgb(base_hsv_color[0], saturation, base_hsv_color[2])
+        return rgb_to_hex((round(val * 255) for val in calculated_hsv_color))
+
+    def _determine_background_color(self, value):
+        if self._is_invalid(value) or self.min_val is None or self.value_range == 0.0:
+            return self.WHITE
+
+        val_ratio = (value - self.min_val) / self.value_range
+
+        if self.hsv_start_color is not None:
+            if val_ratio <= 0.5:
+                # The supplied value should use the start color.
+                # Move the val_ratio back to a [0, 1.0] range with the values flipped.
+                flipped_val_ratio = abs(val_ratio - 0.5) * 2.0
+                saturation = flipped_val_ratio * self.start_saturation_spread
+                return self.get_hex_color_with_new_saturation(self.hsv_start_color, saturation)
+            else:
+                # The supplied value should use the default color.
+                # Adjust it from the (0.5, 1.0] range to (0.0, 1.0]
+                val_ratio = (val_ratio - 0.5) * 2.0
+
+        saturation = val_ratio * self.saturation_spread + 0.05
+        return self.get_hex_color_with_new_saturation(self.hsv_color, saturation)
+
+
+def find_rule_to_apply(rules, value):
     for rule in rules:
-        if color is None:
-            color = rule.apply(value)
+        if rule.applies(value):
+            return rule
 
-    return color
+    return None
 
 
 def map_index_level(index, level, func):
@@ -125,12 +241,12 @@ class ReactTable(Pandas):
                         defaultSortMethod={(a, b, desc) => ReactTableDefaults.defaultSortMethod(a.raw, b.raw, desc)}>
             </ReactTable>;
     """
-
     def __init__(
         self,
         metric,
         *metrics: Field,
         pivot=(),
+        hide=(),
         transpose=False,
         sort=None,
         ascending=None,
@@ -141,15 +257,19 @@ class ReactTable(Pandas):
             metric,
             *metrics,
             pivot=pivot,
+            hide=hide,
             transpose=transpose,
             sort=sort,
             ascending=ascending,
             max_columns=max_columns
         )
         self.formatting_rules_map = defaultdict(list)
+        self.min_max_map = {}
         for formatting_rule in formatting_rules:
-            field_selector = alias_selector(formatting_rule.field.get_alias())
+            field_selector = formatting_rule.get_field_selector()
             self.formatting_rules_map[field_selector].append(formatting_rule)
+            if isinstance(formatting_rule, FormattingHeatMapRule):
+                self.min_max_map[field_selector] = [np.inf, -np.inf]
 
     def __repr__(self):
         return "{}({})".format(
@@ -227,13 +347,16 @@ class ReactTable(Pandas):
         return data_frame
 
     @staticmethod
-    def transform_index_column_headers(data_frame, field_map):
+    def transform_index_column_headers(data_frame, field_map, hide_dimension_aliases):
         """
         Convert the un-pivoted dimensions into ReactTable column header definitions.
 
         :param data_frame:
-            The result set data frame
+            The result set data frame.
         :param field_map:
+            A map to find metrics/operations based on their keys found in the data frame.
+        :param hide_dimension_aliases:
+            A set with hide dimension aliases.
         :return:
             A list of column header definitions with the following structure.
 
@@ -255,6 +378,9 @@ class ReactTable(Pandas):
             return columns
 
         for f_dimension_alias in data_frame.index.names:
+            if f_dimension_alias not in field_map or f_dimension_alias in hide_dimension_aliases:
+                continue
+
             dimension = field_map[f_dimension_alias]
             header = getattr(dimension, "label", dimension.alias)
 
@@ -275,7 +401,7 @@ class ReactTable(Pandas):
 
         :return:
         :param data_frame:
-            The result set data frame
+            The result set data frame.
         :param field_map:
             A map to find metrics/operations based on their keys found in the data frame.
         :return:
@@ -370,11 +496,11 @@ class ReactTable(Pandas):
         return _make_columns(data_frame.columns.to_frame(), dropped_metric_level_name)
 
     @staticmethod
-    def transform_row_index(index_values, field_map, dimension_hyperlink_templates):
+    def transform_row_index(index_values, field_map, dimension_hyperlink_templates, hide_dimension_aliases, row_colors):
         # Add the index to the row
         row = {}
         for key, value in index_values.items():
-            if key is None:
+            if key is None or key not in field_map:
                 continue
 
             field_alias = key
@@ -384,26 +510,50 @@ class ReactTable(Pandas):
             display = _display_value(value, field)
             if display is not None:
                 data["display"] = display
+            if row_colors is not None:
+                data["color"], data["text_color"] = row_colors
+
+            is_totals = display == TOTALS_LABEL
 
             # If the dimension has a hyperlink template, then apply the template by formatting it with the dimension
             # values for this row. The values contained in `index_values` will always contain all of the required values
             # at this point, otherwise the hyperlink template will not be included.
-            if key in dimension_hyperlink_templates:
-                data["hyperlink"] = dimension_hyperlink_templates[key].format(
-                    **index_values
-                )
+            if not is_totals and key in dimension_hyperlink_templates:
+                try:
+                    data["hyperlink"] = dimension_hyperlink_templates[key].format(
+                        **index_values
+                    )
+                except KeyError:
+                    pass
 
             safe_key = safe_value(key)
             row[safe_key] = data
 
+        for dimension_alias in hide_dimension_aliases:
+            del row[dimension_alias]
+
         return row
 
-    def transform_row_values(self, series, fields, is_transposed):
-        # Add the values to the row
+    @staticmethod
+    def _get_row_value_accessor(series, fields, key):
         index_names = series.index.names or []
 
+        accessor_fields = [
+            fields[field_alias]
+            for field_alias in index_names
+            if field_alias is not None
+        ]
+        accessor = [
+            safe_value(value) for value, field in zip(key, accessor_fields)
+        ] or key
+
+        return accessor
+
+    def transform_row_values(self, series, fields, is_transposed, is_pivoted):
         row = {}
-        for key, value in series.iteritems():
+        row_colors = None
+
+        for key, value in series.items():
             key = wrap_list(key)
 
             # Get the field for the metric
@@ -412,41 +562,71 @@ class ReactTable(Pandas):
             data = {
                 RAW_VALUE: raw_value(value, field),
             }
-            color = apply_formatting_rules(self.formatting_rules_map[metric_alias], value)
-            if color is not None:
-                data["color"] = color
+            if not row_colors:
+                # No color for this field yet
+                rule = find_rule_to_apply(self.formatting_rules_map[metric_alias], value)
+                if rule is not None:
+                    colors = rule.determine_colors(value)
+                    data["color"], data["text_color"] = colors
+                    if not is_transposed and not is_pivoted and rule.covers_row:
+                        # No transposing or pivoting going on so set as row color if it's specified for the rule
+                        row_colors = colors
 
             display = _display_value(value, field, date_as=return_none)
             if display is not None:
                 data["display"] = display
 
-            accessor_fields = [
-                fields[field_alias]
-                for field_alias in index_names
-                if field_alias is not None
-            ]
-
-            accessor = [
-                safe_value(value) for value, field in zip(key, accessor_fields)
-            ] or key
-
+            accessor = self._get_row_value_accessor(series, fields, key)
             setdeepattr(row, accessor, data)
 
-        return row
+        # Assign the row colors to fields that aren't colored yet
+        if row_colors:
+            for key in series.keys():
+                accessor = self._get_row_value_accessor(series, fields, wrap_list(key))
+                data = getdeepattr(row, accessor)
+                if "color" not in data:
+                    data["color"], data["text_color"] = row_colors
+
+        return row, row_colors
+
+    def calculate_min_max(self, df, is_transposed):
+        if not self.min_max_map:
+            return
+
+        for index, series in df.iterrows():
+            for key, value in series.items():
+                metric_alias = wrap_list(series.name)[0] if is_transposed else wrap_list(key)[0]
+                min_max = self.min_max_map.get(metric_alias)
+                if min_max is not None:  # we need to calculate the min and max values for this metric.
+                    if value < min_max[0]:
+                        min_max[0] = value  # value is smaller than stored smallest value, replace it.
+                    if value > min_max[1]:
+                        min_max[1] = value  # value is bigger than stored biggest value, replace it.
+
+        for rule_list in self.formatting_rules_map.values():
+            for rule in rule_list:
+                if isinstance(rule, FormattingHeatMapRule):
+                    rule.set_min_max(*self.min_max_map[rule.get_field_selector()])
 
     def transform_data(
-        self, data_frame, field_map, dimension_hyperlink_templates, is_transposed
+        self, data_frame, field_map, hide_aliases, dimension_hyperlink_templates, is_transposed, is_pivoted,
     ):
         """
         Builds a list of dicts containing the data for ReactTable. This aligns with the accessors set by
         #transform_dimension_column_headers and #transform_metric_column_headers
 
         :param data_frame:
-            The result set data frame
+            The result set data frame.
         :param field_map:
             A mapping to all the fields in the dataset used for this query.
+        :param hide_aliases:
+            A set with hide dimension aliases.
         :param dimension_hyperlink_templates:
+            A mapping to fields and its hyperlink dimension, if any.
         :param is_transposed:
+            Whether the table is transposed or not.
+        :param is_pivoted:
+            Whether the table is pivoted or not.
         """
         index_names = data_frame.index.names
 
@@ -468,22 +648,22 @@ class ReactTable(Pandas):
                 axis=1,
             )
 
+        self.calculate_min_max(data_frame, is_transposed)
+
         rows = []
         for index, series in data_frame.iterrows():
-            index = wrap_list(index)
+            row_values, row_colors = self.transform_row_values(series, field_map, is_transposed, is_pivoted)
 
+            index = wrap_list(index)
             # Get a list of values from the index. These can be metrics or dimensions so it checks in the item map if
             # there is a display value for the value
             index_values = (
                 [_get_field_label(value) for value in index] if is_transposed else index
             )
             index_display_values = OrderedDict(zip(index_names, index_values))
-
-            row_index = ReactTable.transform_row_index(
-                index_display_values, field_map, dimension_hyperlink_templates
+            row_index = self.transform_row_index(
+                index_display_values, field_map, dimension_hyperlink_templates, hide_aliases, row_colors
             )
-            row_values = self.transform_row_values(series, field_map, is_transposed)
-
             rows.append(
                 {
                     **row_index,
@@ -506,11 +686,11 @@ class ReactTable(Pandas):
         `data` which align with the props in ReactTable with the same name.
 
         :param data_frame:
-            The result set data frame
+            The result set data frame.
         :param dimensions:
-            A list of dimensions that were selected in the data query
+            A list of dimensions that were selected in the data query.
         :param references:
-            A list of references that were selected in the data query
+            A list of references that were selected in the data query.
         :param annotation_frame:
             A data frame containing the annotation data.
         :param use_raw_values:
@@ -519,6 +699,12 @@ class ReactTable(Pandas):
             An dict containing attributes `columns` and `data` which align with the props in ReactTable with the same
             names.
         """
+        result_df = data_frame.copy()
+
+        dimension_map = {
+            alias_selector(dimension.alias): dimension for dimension in dimensions
+        }
+
         metric_map = OrderedDict(
             [
                 (
@@ -529,9 +715,7 @@ class ReactTable(Pandas):
                 for ref in [None] + references
             ]
         )
-        dimension_map = {
-            alias_selector(dimension.alias): dimension for dimension in dimensions
-        }
+
         field_map = {
             **metric_map,
             **dimension_map,
@@ -544,24 +728,34 @@ class ReactTable(Pandas):
                 METRICS_DIMENSION_ALIAS, None, data_type=DataType.text, label=""
             ),
         }
-        # has at least 1 dim and all are pivoted
-        all_dimensions_pivoted = len(dimensions) and (len(dimensions) == len(self.pivot))
-
         metric_aliases = list(metric_map.keys())
-        dimension_aliases = [
-            alias_selector(dimension.alias) for dimension in self.pivot
+
+        hide_aliases = {
+            alias_selector(dimension.alias) for dimension in self.hide
+        }
+
+        for dimension in dimensions:
+            if dimension.fetch_only:
+                hide_aliases.add(alias_selector(dimension.alias))
+
+        pivot_dimensions = [
+            alias_selector(dimension.alias)
+            for dimension in self.pivot
+            if alias_selector(dimension.alias) not in hide_aliases
         ]
 
-        df = self.format_data_frame(data_frame[metric_aliases])
-        df = self.pivot_data_frame(df, dimension_aliases, self.transpose)
+        result_df = self.format_data_frame(result_df[metric_aliases])
+        result_df, is_pivoted, is_transposed = self.pivot_data_frame(result_df, pivot_dimensions, self.transpose)
+        dimension_columns = self.transform_index_column_headers(result_df, field_map, hide_aliases)
+        metric_columns = self.transform_data_column_headers(result_df, field_map)
 
-        dimension_columns = self.transform_index_column_headers(df, field_map)
-        metric_columns = self.transform_data_column_headers(df, field_map)
         data = self.transform_data(
-            df,
+            result_df,
             field_map,
-            is_transposed=self.transpose ^ all_dimensions_pivoted,
-            dimension_hyperlink_templates=self.map_hyperlink_templates(df, dimensions),
+            hide_aliases=hide_aliases,
+            dimension_hyperlink_templates=self.map_hyperlink_templates(result_df, dimensions),
+            is_transposed=is_transposed,
+            is_pivoted=is_pivoted,
         )
 
         return {"columns": dimension_columns + metric_columns, "data": data}
